package dict

import (
	"os"
	"reflect"
	"testing"

	"gopkg.in/yaml.v3"
)

type dictDictToDictTestCase struct {
	Dict1  Dict `yaml:"dict1"`
	Dict2  Dict `yaml:"dict2"`
	Result Dict `yaml:"result"`
}

type dictStrToDictTestCase struct {
	Name   string `yaml:"name"`
	Arg1   Dict   `yaml:"arg1"`
	Arg2   string `yaml:"arg2"`
	Result Dict   `yaml:"result"`
}

func loadYamlFile(fn string, out interface{}) {
	content, err := os.ReadFile(fn)
	if err != nil {
		panic(err)
	}

	err = yaml.Unmarshal(content, out)
	if err != nil {
		panic(err)
	}
}

func sampleDict() Dict {
	return Dict{
		"name":        "Test Project",
		"version":     1.0,
		"active":      true,
		"description": "This is a sample YAML file for testing the Dict package.",
		"metadata": Dict{
			"author":  "John Doe",
			"license": "MIT",
			"tags":    []interface{}{"sample", "test", "yaml"},
		},
		"configuration": Dict{
			"threads":  4,
			"verbose":  true,
			"features": []interface{}{"logging", "monitoring"},
		},
		"resources": []interface{}{
			Dict{
				"type": "database",
				"name": "primary-db",
				"properties": Dict{
					"host": "localhost",
					"port": 5432,
					"credentials": Dict{
						"username": "admin",
						"password": "secret",
					},
				},
			},
			Dict{
				"type": "cache",
				"name": "redis-cache",
				"properties": Dict{
					"host": "localhost",
					"port": 6379,
					"ttl":  3600,
				},
			},
		},
		"nested": Dict{
			"level1": Dict{
				"level2": Dict{
					"level3": Dict{
						"key":   "value",
						"array": []interface{}{1, 2, 3, 4.5},
						"nestedArray": []interface{}{
							Dict{"key1": "val1"},
							Dict{
								"key2": "val2",
								"subNested": Dict{
									"subKey1": "subVal1",
									"subKey2": 2,
								},
							},
						},
					},
				},
			},
		},
		"additionalNotes": "This is a multiline string.\nIt can contain new lines and spaces.\nIt's useful for longer descriptions or content.",
	}
}

func TestSampleDict(t *testing.T) {
	originalDict, err := LoadYaml("test_data/dict2.yaml")
	if err != nil {
		t.Fatalf("LoadYaml() returned an error: %v", err)
	}

	dict := sampleDict()

	if !reflect.DeepEqual(dict, originalDict) {
		t.Errorf("Expected dict to be %v, got %v", originalDict, dict)
	}

}
